//+------------------------------------------------------------------+
//|                                                     Try This.mq4 |
//|                                              AK Copyright  2009 |
//|                                                                  |
//+------------------------------------------------------------------+

#define MAGICNO  20050610

extern double Lots = 0.1;
extern int SetUpLength = 3;
extern bool EmergencyStopLoss = FALSE;
extern double StopLossLevel = 1.618;

int FirstCross; //Variable use to check whether the MACD has just crossed
int LastAlert; //Variable to ensure signal is only given once, ie, don't keep getting buy signals on the
               //same bar on each new tick
int BuyCount = 100; //Assigned when criteria is met then, tested to see whether it's within the SetUpLength range so a
                    //buy signal can be given.  Initialsed to 100 so it's over the SetUpLength range
int SellCount = 100; //Same as above but for sell signals


//Function to see whether it's the beginning of a bar
bool NewBar()
{
   static datetime dt = 0;
   if (dt != Time[0])
   {
      dt = Time[0]; Sleep(100); // wait for tick
      return(true);
   }
   return(false);
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
   double MACD, MACDPrevious, BolLow, BolHigh, StopLoss;
   int Total, Ticket, Count;

   if(Bars<100)
   {
      Print("bars less than 100");
      return(0);  
   }

   MACD = iMACD(Symbol(),0,2,6,12,PRICE_CLOSE,MODE_SIGNAL,1); //MACD for bar previous to the current bar on the chart
   MACDPrevious = iMACD(Symbol(),0,2,6,12,PRICE_CLOSE,MODE_SIGNAL,2); //MACD a bar before MACD above
   BolLow = iBands(Symbol(),0,20,2,0,PRICE_CLOSE,MODE_LOWER,1);
   BolHigh = iBands(Symbol(),0,20,2,0,PRICE_CLOSE,MODE_UPPER,1);
   
   if (NewBar())  //Ensures that this only happens once and at the beginning of the bar
   {
      if ((MACDPrevious <= 0 && MACD >= 0) || (MACDPrevious >= 0 && MACD <= 0)) //Test to see whether MACD has just crossed
      {
         FirstCross = 1;
         LastAlert = 0;  
      }
   }
   
   Total=OrdersTotal();
   if(Total<1) 
   {
      if (AccountFreeMargin()<(1000*Lots))
      {
         Print("We Have No Money. Free Margin = ", AccountFreeMargin());
         return(0);  
      }
      
      if (FirstCross == 1) //Only want to look for signals if the MACD has just crossed and onwards
      {
         Print("First Cross Is: ", FirstCross, ", Alert Is: ", LastAlert);
         Print("Buy Count Is: ", BuyCount, ", Sell Count Is: ", SellCount);
            if (Open[0] <= BolLow) //Buy Signal
            {
               if (LastAlert != 2) //Checks to make sure we haven't had a buy signal yet
               {
                  Print("Buy Signal");
                  BuyCount = 0; //Initialises to 0 to say we have a buy signal
                  FirstCross = 0; //Initialises to 0 so that we don't test for signals anymore, only looking for trades now
               }
               LastAlert = 2; //After testing, regardless of finding a signal or not, we set to 2 so that we don't get 
                              //anymore buy signals till we enter a trade or the MACD crosses over again
            }
            if (Open[0] >= BolHigh) //Sell Signal
            {
               if (LastAlert != 1)
               {
                  Print("Sell Signal");
                  SellCount = 0;
                  FirstCross = 0;
               }
               LastAlert = 1;
            }
      }
               
      if (BuyCount <= SetUpLength) //Tests whether we got a buy signal and it's within the user defined length of bars
      {
         if (Ask > High[BuyCount]) //Make sure the price action has broken the high of the signal bar
            {
               if (EmergencyStopLoss) //Adds a stoploss if user defined true, else 0
               {
                  StopLoss = Bid - ((High[BuyCount] - Low[BuyCount]) * StopLossLevel);
               }
               else StopLoss = 0;
               Print("Enter Trade");
               Ticket=OrderSend(Symbol(),OP_BUY,Lots,Ask,3,StopLoss,0,"ADV Buy",MAGICNO,0,Blue);
               Print("Trade Entered");
               BuyCount = 100; //Sets back to 100 once the trade is sent so we don't get more trades, incase we are closed
                               //on the same bar
               return(0);
            }
            else
            {
               if (NewBar()) BuyCount++; //At the beginning of the bar, increment the bar counter, ie, we're on the next bar
               Print("First Cross Is: ", FirstCross, ", Alert Is: ", LastAlert);
               Print("Buy Count Is: ", BuyCount, ", Sell Count Is: ", SellCount);
               return(0);
            }
      }
      else BuyCount = 100; //If we're outside the range, make it 100, so it's way away from the range and no chance of
                           //accidentally taking a trade
      
      //Sell is exactly the same, except we;re waiting for price action to break the low of the signal bar 
      if (SellCount <= SetUpLength)
      {
         if (Bid < Low[SellCount])
            {
               if (EmergencyStopLoss)
               {
                  StopLoss = Ask + ((High[SellCount] - Low[SellCount]) * StopLossLevel);
               }
               else StopLoss = 0;
               Print("Enter Trade");
               Ticket=OrderSend(Symbol(),OP_SELL,Lots,Bid,3,StopLoss,0,"ADV Sell",MAGICNO,0,Red);
               Print("Trade Entered");
               SellCount = 100;
               return(0);
            }
            else
            {
               if (NewBar()) SellCount++;
               Print("First Cross Is: ", FirstCross, ", Alert Is: ", LastAlert);
               Print("Buy Count Is: ", BuyCount, ", Sell Count Is: ", SellCount);
               return(0);
            }
      }
      else SellCount = 100;
      
      Print("First Cross Is: ", FirstCross, ", Alert Is: ", LastAlert);
      Print("Buy Count Is: ", BuyCount, ", Sell Count Is: ", SellCount);
      
      return(0);
   }
   
   for(Count=0;Count<Total;Count++)
   {
      Print("First Cross Is: ", FirstCross, ", Alert Is: ", LastAlert);
      Print("Buy Count Is: ", BuyCount, ", Sell Count Is: ", SellCount);
      OrderSelect(Count, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()<=OP_SELL && OrderSymbol()==Symbol())
      {
         if(OrderType()==OP_BUY) //Close A Long Position
         {
            if(Open[0] >= BolHigh)
            {
               OrderClose(OrderTicket(),OrderLots(),Bid,3,Violet);
               Print("Trade Closed");
               FirstCross = 0; //Set to 0 so we have to wait for the MACD to cross again before taking another trade
                               //otherwise it would open a trade straight away
               return(0);
            }
         }
         else
         {
            if(Open[0] <= BolLow) //Close A Short Position
            {
               OrderClose(OrderTicket(),OrderLots(),Bid,3,Violet);
               Print("Trade Closed");
               FirstCross = 0;
               return(0);
            }
         }
      }
   }
   return(0);
}
//+------------------------------------------------------------------+